<?php

/* * ******************************************************************
 * Version 1.0
 * Modified: 29-08-2015
 * Copyright 2015 Accentio. All rights reserved.
 * License: None
 * By: Michel Jongbloed
 * ****************************************************************** */

// Prevent direct access
if ( ! defined ( 'ABSPATH' ) ) {
    echo __ ( 'Hi!  I\'m just a plugin, there\'s not much I can do when called directly.', 'wppfm-strings' );
    exit;
}


if ( ! class_exists ( 'WPPFM_Feed_Table' ) ) :

    /**
     * The WPPFM_Feed_Table class extends a copy of the WordPress core WP_List_Table and is used to place a standard
     * table on a form.
     * 
     * 
     * 
     * @class WPPFM_Feed_Table
     * @version dev
     */
    class WPPFM_Feed_Table extends WPC_List_Table {

        /**
         * Column names
         */
        private $columns_array = array ();

        /**
         * Sortable columns
         */
        private $sortable_columns = array ();

        /**
         * Max items per page
         */
        private $items_per_page = 15;

        /**
         * Class constructor
         */
        function __construct () {

            parent::__construct ( array (
                'singular' => 'wp_list_text_link', //Singular label
                'plural' => 'wp_list_test_links', //plural label, also this well be one of the table css class
                'ajax' => false //We won't support Ajax for this table
            ) );
        }

        /**
         * Extra markup before and after the table
         * 
         * @param string $which
         */
        function extra_tablenav ( $which ) {

            if ( $which == "top" ) {

                //The code that goes before the table is here
                echo "Hello, I'm before the table";
            }

            if ( $which == "bottom" ) {

                //The code that goes after the table is there
                echo "Hi, I'm after the table";
            }
        }

        /**
         * Get the max number of items per page.
         * 
         * @return int
         */
        public function get_max_items_per_page () {

            return $this -> items_per_page;
        }

        /**
         * Set the max number of items per page
         * 
         * @param int max number of items per page
         */
        public function set_max_items_per_page ( $value ) {

            $this -> items_per_page = $value;
        }

        /**
         * Set the columns titles
         * 
         * @param array with strings of column names
         */
        public function set_columns ( $column_titles ) {

            $this -> columns_array = $column_titles;
        }

        /**
         * Defines the sortable columns
         * 
         * @param array with columns that need to be sortable
         */
        public function set_sortable_columns ( $sortable_columns ) {

            $this -> sortable_columns = $sortable_columns;
        }

        /**
         * Make some preparations for the table
         */
        function prepare_items () {

            global $wpdb;
            global $_wp_column_headers;

            $screen = get_current_screen ();

            // the query
            $query = "SELECT * FROM $wpdb -> links";

            // how do we want to order things
            $orderby = ! empty ( $_GET [ "orderby" ] ) ? mysql_real_escape_string ( $_GET [ "orderby" ] ) : 'ASC';
            $order = ! empty ( $_GET [ "order" ] ) ? mysql_real_escape_string ( $_GET [ "order" ] ) : '';

            if ( ! empty ( $orderby ) & ! empty ( $order ) ) {

                $query .= ' ORDER BY ' . $orderby . ' ' . $order;
            }

            // pagination parameters
            $totalitems = $wpdb -> query ( $query ); //return the total number of affected rows
            $perpage = $this -> items_per_page;
            $paged = ! empty ( $_GET [ "paged" ] ) ? mysql_real_escape_string ( $_GET [ "paged" ] ) : ''; // which page are we on
            // current page number
            if ( empty ( $paged ) || ! is_numeric ( $paged ) || $paged <= 0 ) {

                $paged = 1;
            }

            // total pages
            $totalpages = ceil ( $totalitems / $perpage );

            // adjust the query to take pagination into account
            if ( ! empty ( $paged ) && ! empty ( $perpage ) ) {

                $offset = ( $paged - 1 ) * $perpage;
                $query .= ' LIMIT ' . ( int ) $offset . ',' . ( int ) $perpage;
            }

            // now register the pagination
            $this -> set_pagination_args ( array (
                "total_items" => $totalitems,
                "total_pages" => $totalpages,
                "per_page" => $perpage,
            ) );

            // register the columns
            $hidden = array ();
            //$columns = $this->get_columns ();
            $columns = $this -> columns_array;
            $sortable = $this -> sortable_columns;
            $this -> _column_headers = array ( $columns, $hidden, $sortable );

            // go get the results
            $this -> items = $wpdb -> get_results ( $query );
        }

        /**
         * Display the rows of records 
         * 
         * @return string
         */
        function display_rows () {
            // get the records registered in the prepare_items method
            $records = $this -> items;

            // get the columns registered in the get_columns and get_sortable_columns methods
            list( $columns, $hidden ) = $this -> get_column_info ();

            // walk through the records
            if ( ! empty ( $records ) ) {

                foreach ( $records as $rec ) {

                    echo '<tr id="record_' . $rec -> link_id . '">';

                    foreach ( $columns as $column_name => $column_display_name ) {

                        // style attributes for each column
                        $class = "class='$column_name column-$column_name'";
                        $style = "";

                        if ( in_array ( $column_name, $hidden ) ) {
                            $style = ' style="display:none;"';
                        }

                        $attributes = $class . $style;

                        $editlink = '/wp-admin/link.php?action=edit&link_id=' . ( int ) $rec -> link_id;

                        // display the specific cell
                        switch ( $column_name ) {
                            case "col_link_id": echo '<td ' . $attributes . '>' . stripslashes ( $rec -> link_id ) . '</td>';
                                break;
                            case "col_link_name": echo '<td ' . $attributes . '><strong><a href="' . $editlink . '" title="Edit">' . stripslashes ( $rec -> link_name ) . '</a></strong></td>';
                                break;
                            case "col_link_url": echo '<td ' . $attributes . '>' . stripslashes ( $rec -> link_url ) . '</td>';
                                break;
                            case "col_link_description": echo '<td ' . $attributes . '>' . $rec -> link_description . '</td>';
                                break;
                            case "col_link_visible": echo '<td ' . $attributes . '>' . $rec -> link_visible . '</td>';
                                break;
                        }
                    }

                    // close the table
                    echo '</tr>';
                }
            }
        }

    }

     // end of WPPFM_Feed_Table class

endif;