<?php

/* * ******************************************************************
 * Version 2.0
 * Modified: 11-05-2016
 * Copyright 2016 Accentio. All rights reserved.
 * License: None
 * By: Michel Jongbloed
 * ****************************************************************** */

// Prevent direct access
if ( !defined( 'ABSPATH' ) ) {
	echo __( 'Hi!  I\'m just a plugin, there\'s not much I can do when called directly.', 'wppfm-strings' );
	exit;
}

if ( !class_exists( 'WPPFM_Schedules' ) ) :

	/**
	 * The WPPFM_Schedules class contains the functions that perform scheduled tasks like updating the active feeds
	 * 
	 * @class WPPFM_Schedules
	 * @version dev
	 */
	class WPPFM_Schedules {

		public function update_active_feeds( $silent = false ) {

			$data_class			 = new WPPFM_Data_Class();

			$current_timestamp = date( 'Y-m-d H:i:s' );

			$active_feeds_schedules = $data_class->get_schedule_data();

			foreach ( $active_feeds_schedules as $schedule ) {

				$update_time = $this->get_new_activation_time( $schedule[ 'updated' ], $schedule[ 'schedule' ] );

				// activate the feed update when the update time is reached
				if ( $update_time < $current_timestamp ) {

					// $feed_data should be in the same format as the data that comes from the javascript _feedHolder
					// the function get_feed_data fetches the data from the database and converts it to the same
					// format as the _feedHolder has.
					$feed_data = $data_class->get_feed_data( $schedule[ 'product_feed_id' ] );

					$feed_master_class	 = new WPPFM_Feed_Master_Class($feed_data);

					$feed_master_class->update_feed_file( $feed_data, $silent );
				}
			}
		}

		private function get_new_activation_time( $last_update, $update_frequency ) {

			$update_split = explode( ':', $update_frequency );

			$update_date		 = date_add( date_create( $last_update ), date_interval_create_from_date_string( $update_split[ 0 ] . ' days' ) );
			$update_date_time	 = date_format( $update_date, 'Y-m-d' ) . ' ' . $update_split[ 1 ] . ':' . $update_split[ 2 ] . ':00';

			return $update_date_time;
		}

	}

	

    

    

	

	

     // end of WPPFM_Schedules class

endif;