<?php

/* * ******************************************************************
 * Version 1.0
 * Modified: 29-08-2015
 * Copyright 2015 Accentio. All rights reserved.
 * License: None
 * By: Michel Jongbloed
 * ****************************************************************** */

// Prevent direct access
if ( !defined( 'ABSPATH' ) ) {
	echo __( 'Hi!  I\'m just a plugin, there\'s not much I can do when called directly.', 'wppfm-strings' );
	exit;
}

if ( !class_exists( 'WPPFM_Ajax_File_Class' ) ) :

	/**
	 * The WPPFM_Ajax_File_Class contains all functions for file manipulation ajax calls
	 * 
	 * @class WPPFM_Ajax_File_Class
	 * @version dev
	 */
	class WPPFM_Ajax_File_Class extends WPPFM_Ajax_Calls {

		/**
		 * Class constructor
		 */
		public function __construct() {

			global $wp_filesystem;

			parent::__construct();

			$this->_queries			 = new WPPFM_Queries ();
			$this->_wp_filesystem	 = &$wp_filesystem;

			// hooks
			add_action( 'wp_ajax_myajax-get-next-categories', array( $this, 'myajax_read_next_categories' ) );
			add_action( 'wp_ajax_myajax-get-category-lists', array( $this, 'myajax_read_category_lists' ) );
			add_action( 'wp_ajax_myajax-delete-feed-file', array( $this, 'myajax_delete_feed_file' ) );
			add_action( 'wp_ajax_myajax-update-feed-file', array( $this, 'myajax_update_feed_file' ) );
			add_action( 'wp_ajax_myajax-log-message', array( $this, 'myajax_log_message' ) );
			add_action( 'wp_ajax_myajax-update-ftp-mode-selection', array( $this, 'myajax_update_ftp_mode_selection' ) );
		}

		/* --------------------------------------------------------------------------------------------------*
		 * Public functions
		 * -------------------------------------------------------------------------------------------------- */

		public function myajax_read_next_categories() {

			// make sure this call is legal
			if ( $this->safe_ajax_call( filter_input( INPUT_POST, 'nextCategoryNonce' ), 'myajax-next-category-nonce' ) ) {

				$channel_id		 = filter_input( INPUT_POST, 'channelId' );
				$requested_level = filter_input( INPUT_POST, 'requestedLevel' );
				$parent_category = filter_input( INPUT_POST, 'parentCategory' );
				$file_language	 = filter_input( INPUT_POST, 'fileLanguage' );

				$file_class = new WPPFM_File_Class();

				$categories = $file_class->get_categories_for_list( $channel_id, $requested_level, $parent_category, $file_language );

				if ( !is_array( $categories ) ) {
					if ( substr( $categories, -1 ) === "0" ) {
						chop( $categories, '0' );
					}
				}

				echo json_encode( $categories );
			}

			// IMPORTANT: don't forget to exit
			exit;
		}

		public function myajax_read_category_lists() {

			// make sure this call is legal
			if ( $this->safe_ajax_call( filter_input( INPUT_POST, 'categoryListsNonce' ), 'myajax-category-lists-nonce' ) ) {

				$channel_id				 = filter_input( INPUT_POST, 'channelId' );
				$main_categories_string	 = filter_input( INPUT_POST, 'mainCategories' );
				$file_language			 = filter_input( INPUT_POST, 'fileLanguage' );

				$file_class = new WPPFM_File_Class();

				$categories_array = explode( '>', $main_categories_string );

				$categories = array();

				for ( $i = 0; $i < count( $categories_array ); $i ++ ) {

					$parent_category = $i > 0 ? $categories_array[ $i - 1 ] : '';

					$c = $file_class->get_categories_for_list( $channel_id, $i, $parent_category, $file_language );

					array_push( $categories, $c );
				}

				echo json_encode( $categories );
			}

			// IMPORTANT: don't forget to exit
			exit;
		}

		public function myajax_delete_feed_file() {

			// make sure this call is legal
			if ( $this->safe_ajax_call( filter_input( INPUT_POST, 'deleteFeedNonce' ), 'myajax-delete-feed-nonce' ) ) {

				$file_name = filter_input( INPUT_POST, 'fileTitle' );
				
				if ( file_exists( WP_PLUGIN_DIR . '/wp-product-feed-manager-support/feeds/' . $file_name ) ) {
					$file = WP_PLUGIN_DIR . '/wp-product-feed-manager-support/feeds/' . $file_name;
				} else {
					$file = WPPFM_FEEDS_DIR . '/' . $file_name;
				}

				// only return results when the user is an admin with manage options
				if ( is_admin() && current_user_can( 'manage_options' ) ) {
					echo file_exists( $file ) ? unlink( $file ) : wppfm_show_wp_error( __( "Could not find file $file.", 'wppfb-strings' ) );
				} else {
					echo wppfm_show_wp_error( __( 'Error deleting the feed. You do not have the correct authorities to delete the file.', 'wppfb-strings' ) );
				}
			}

			// IMPORTANT: don't forget to exit
			exit;
		}

		/**
		 * This function fetches the posted data and triggers the update of the feed file on the server.
		 * 
		 * @param: nothing
		 * @return callback echo containing the result of the update
		 */
		public function myajax_update_feed_file() {

			// make sure this call is legal
			if ( $this->safe_ajax_call( filter_input( INPUT_POST, 'updateFeedFileNonce' ), 'myajax-update-feed-file-nonce' ) ) {

				// fetch the data from $_POST
				$feed_received	 = filter_input( INPUT_POST, 'feedData' );
				$feed_data		 = json_decode( $feed_received );

				$feed_master_class = new WPPFM_Feed_Master_Class($feed_data);

				// only return results when the user is an admin with manage options
				if ( is_admin() && current_user_can( 'manage_options' ) ) {

					$feed_master_class->update_feed_file( $feed_data, false );
				} else {

					echo wppfm_show_wp_error( __( 'Error writing the feed. You do not have the correct authorities to write the file.', 'wppfb-strings' ) );
				}
			}

			// IMPORTANT: don't forget to exit
			exit;
		}

		public function myajax_log_message() {

			// make sure this call is legal
			if ( $this->safe_ajax_call( filter_input( INPUT_POST, 'logMessageNonce' ), 'myajax-log-message-nonce' ) ) {

				// fetch the data from $_POST
				$message		 = filter_input( INPUT_POST, 'messageList' );
				$fileName		 = filter_input( INPUT_POST, 'fileName' );
				$text_message	 = strip_tags( $message );

				// only return results when the user is an admin with manage options
				if ( is_admin() && current_user_can( 'manage_options' ) ) {

					echo wppfm_write_log_file( $text_message, $fileName );
				} else {

					echo wppfm_show_wp_error( __( 'Error writing the feed. You do not have the correct authorities to write the file.', 'wppfb-strings' ) );
				}
			}

			// IMPORTANT: don't forget to exit
			exit;
		}
	
		public function myajax_update_ftp_mode_selection() {
			
			// make sure this call is legal
			if ( $this->safe_ajax_call( filter_input( INPUT_POST, 'updateFeedDataNonce' ), 'myajax-set-ftp-mode-nonce' ) ) {

				$selection = filter_input( INPUT_POST, 'ftp_selection' );
				
				update_option( 'wppfm_ftp_passive', $selection );
				
				echo get_option( 'wppfm_ftp_passive' );
			}

			// IMPORTANT: don't forget to exit
			exit;
		}

	}

	// End of WPPFM_Ajax_File_Class

endif;

$myajaxfileclass = new WPPFM_Ajax_File_Class();
