<?php

/* * ******************************************************************
 * Version 3.0
 * Modified: 19-12-2016
 * Copyright 2016 Accentio. All rights reserved.
 * License: None
 * By: Michel Jongbloed
 * ****************************************************************** */

// Prevent direct access
if ( !defined( 'ABSPATH' ) ) {
	echo __( 'Hi!  I\'m just a plugin, there\'s not much I can do when called directly.', 'wppfm-strings' );
	exit;
}

if ( !class_exists( 'WPPFM_Queries' ) ) :

	/**
	 * The WPPFM_Queries Class contains the database queries
	 * 
	 * @class WPPFM_Queries
	 * @version dev
	 * @category class
	 */
	class WPPFM_Queries {

		/**
		 * @var class reference
		 */
		private $_wpdb;
		
		/**
		 * @var string placeholder containing the wp table prefix
		 */
		private $_table_prefix;

		/**
		 * WPPFM_Queries Constructor
		 */
		public function __construct() {

			// get global wordpress database functions
			global $wpdb;

			// assign the global wpdb to a varable
			$this->_wpdb = &$wpdb;

			// assign the wp table prefix to a variable
			$this->_table_prefix = $this->_wpdb->prefix;
		}

		public function make_list_of_active_feeds() {

			$main_table		 = $this->_table_prefix . 'feedmanager_product_feed';
			$status_table	 = $this->_table_prefix . 'feedmanager_feed_status';

			return $this->_wpdb->get_results( "SELECT p.product_feed_id, p.title, p.url, p.updated, p.products, s.status AS status, s.color AS color "
			. "FROM $main_table AS p "
			. " INNER JOIN $status_table AS s on p.status_id = s.status_id" );
		}
		
		public function get_feed_row( $feed_id ) {
			
			$main_table		 = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->get_row( "SELECT * FROM $main_table WHERE product_feed_id = $feed_id" );
		}

		/**
		 * Get a list of all existing countries
		 * 
		 * @return results of the query
		 */
		public function read_countries() {

			$main_table = $this->_table_prefix . 'feedmanager_country';
			return $this->_wpdb->get_results( "SELECT name_short, name FROM $main_table ORDER BY name", ARRAY_A );
		}

		public function read_channels() {
			$google = array( 'channel_id'=>'1', 'name'=>'Google Merchant', 'short'=>'google' );
			return array( $google ); //NOG NADENKEN => Over hoe ik het zo kan aanpassen dat Google automatisch wordt geselecteerd!
		}

		public function register_a_channel( $channel_short_name, $channel_id, $channel_name ) {

			$main_table = $this->_table_prefix . 'feedmanager_channel';
			return $this->_wpdb->query( "INSERT INTO $main_table (channel_id, name, short) VALUES
				( $channel_id, '$channel_name', '$channel_short_name' )" );
		}

		public function get_channel_id( $channel_short_name ) {

			$main_table = $this->_table_prefix . 'feedmanager_channel';
			return $this->_wpdb->get_var( "SELECT channel_id FROM $main_table WHERE short = '$channel_short_name'" );
		}

		public function get_channel_short_name_from_db( $channel_id ) {

			if ( $channel_id !== 'undefined' ) { // make sure the selected channel is installed
				$main_table = $this->_table_prefix . 'feedmanager_channel';
				return $this->_wpdb->get_var( "SELECT short FROM $main_table WHERE channel_id = $channel_id" );
			} else { return false; }
		}

		public function remove_channel( $channel_id ) {

			$main_table = $this->_table_prefix . 'feedmanager_channel';
			return $this->_wpdb->delete( $main_table, array( 'channel_id' => $channel_id ) );
		}

		public function read_active_schedule_data() {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->get_results( "SELECT product_feed_id, updated, schedule FROM $main_table WHERE status_id=1", ARRAY_A );
		}

		public function read_sources() {

			$main_table = $this->_table_prefix . 'feedmanager_source';
			return $this->_wpdb->get_results( "SELECT source_id, name FROM $main_table ORDER BY name", ARRAY_A );
		}

		/**
		 * Reads the standard output fields that are specific for the specified channel
		 * 
		 * @access public
		 * @param type $channel_id
		 * @param type $language
		 * @return result of the query
		 */
// deprecated 271216
//		public function read_output_fields( $channel_id, $language ) {
//
//			$channel_id = unknown;
//
//			$main_table	 = $this->_table_prefix . 'feedmanager_field_to_channel';
//			$link_table	 = $this->_table_prefix . 'feedmanager_field_' . $language;
//
//			return $this->_wpdb->get_results( "SELECT p.field_id, p.category_id, l.field_label "
//			. "FROM $main_table AS p "
//			. "INNER JOIN $link_table AS l on p.field_id = l.field_id "
//			. "WHERE p.channel_id = $channel_id" );
//		}

		public function get_feeds_from_specific_channel( $channel_id ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->get_results( "SELECT product_feed_id FROM $main_table WHERE channel_id = $channel_id", ARRAY_A );
		}

		public function get_meta_parents( $feed_id ) {

			$main_table = $this->_table_prefix . 'posts';
			return $this->_wpdb->get_results( "SELECT ID FROM $main_table WHERE post_parent = $feed_id", ARRAY_A );
		}

		public function read_feed( $feed_id ) {

			$main_table		 = $this->_table_prefix . 'feedmanager_product_feed';
			$countries_table = $this->_table_prefix . 'feedmanager_country';
			$channel_table	 = $this->_table_prefix . 'feedmanager_channel';

			$result = $this->_wpdb->get_results( "SELECT p.product_feed_id, p.source_id AS source, p.title, p.main_category, "
			. "p.url, p.include_variations, p.is_aggregator, p.status_id, p.updated, p.schedule, c.name_short "
			. "AS country, m.channel_id AS channel, p.status_id "
			. "FROM $main_table AS p "
			. "INNER JOIN $countries_table AS c ON p.country_id = c.country_id "
			. "INNER JOIN $channel_table AS m ON p.channel_id = m.channel_id "
			. "WHERE p.product_feed_id = $feed_id", ARRAY_A );

			$category_mapping = $this->read_category_mapping( $feed_id );

			if ( isset( $category_mapping[ 0 ][ 'meta_value' ] ) && $category_mapping[ 0 ][ 'meta_value' ] !== '' ) {
				$result[ 0 ][ 'category_mapping' ] = $category_mapping[ 0 ][ 'meta_value' ];
			} else {
				$result[ 0 ][ 'category_mapping' ] = '';
			}

			return $result;
		}
		
		public function read_category_mapping( $feed_id ) {

			$meta_table		 = $this->_table_prefix . 'feedmanager_product_feedmeta';
			return $this->_wpdb->get_results( "SELECT meta_value FROM $meta_table WHERE product_feed_id = $feed_id AND meta_key = 'category_mapping'", ARRAY_A );
		}

		public function read_post_data( $column_string, $category_string, $offset, $limit ) {

			$main_table					 = $this->_table_prefix . 'posts';
			$term_relationships_table	 = $this->_table_prefix . 'term_relationships';
			$term_taxonomy_table		 = $this->_table_prefix . 'term_taxonomy';
			$selecting_columns			 = $column_string ? ', ' . $column_string : '';

			return $this->_wpdb->get_results( "SELECT DISTINCT ID $selecting_columns 
				FROM $main_table 
				LEFT JOIN $term_relationships_table ON ($main_table.ID = $term_relationships_table.object_id) 
				LEFT JOIN $term_taxonomy_table ON ($term_relationships_table.term_taxonomy_id = $term_taxonomy_table.term_taxonomy_id)
				WHERE $main_table.post_status = 'publish'
				AND ID >= $offset AND ID < $limit
				AND $term_taxonomy_table.term_id IN ($category_string)
				ORDER BY ID" );
		}

		/**
		 * returns the lowest product id belonging to one or more categories
		 * 
		 * @param string $category_string
		 * @return string
		 */
		public function get_lowest_product_id( $category_string ) {
			
			$main_table	= $this->_table_prefix . 'posts';
			$term_relationships_table	 = $this->_table_prefix . 'term_relationships';
			$term_taxonomy_table = $this->_table_prefix . 'term_taxonomy';

			$result = $this->_wpdb->get_results( "SELECT MIN(ID) FROM $main_table 
				LEFT JOIN $term_relationships_table ON ($main_table.ID = $term_relationships_table.object_id) 
				LEFT JOIN $term_taxonomy_table ON ($term_relationships_table.term_taxonomy_id = $term_taxonomy_table.term_taxonomy_id)
				WHERE post_type = 'product' AND post_status = 'publish'
				AND $term_taxonomy_table.term_id IN ($category_string)"
				, ARRAY_A );

			return $result ? $result[ 0 ][ 'MIN(ID)' ] : 0;
		}
		

		/**
		 * returns the highest product id belonging to one or more categories
		 * 
		 * @param string $category_string
		 * @return string
		 */
		public function get_highest_product_id( $category_string ) {
			
			$main_table	= $this->_table_prefix . 'posts';
			$term_relationships_table	 = $this->_table_prefix . 'term_relationships';
			$term_taxonomy_table = $this->_table_prefix . 'term_taxonomy';

			$result = $this->_wpdb->get_results( "SELECT MAX(ID) FROM $main_table 
				LEFT JOIN $term_relationships_table ON ($main_table.ID = $term_relationships_table.object_id) 
				LEFT JOIN $term_taxonomy_table ON ($term_relationships_table.term_taxonomy_id = $term_taxonomy_table.term_taxonomy_id)
				WHERE post_type = 'product' AND post_status = 'publish'
				AND $term_taxonomy_table.term_id IN ($category_string)"
				, ARRAY_A );

			return $result ? $result[ 0 ][ 'MAX(ID)' ] : 0;
		}

		// deprecated 271216
//		public function read_post_column( $post_id, $column_name ) {
//
//			$main_table = $this->_table_prefix . 'posts';
//			//$result = $this->_wpdb->get_results( "SELECT $column_name FROM $main_table WHERE ID=$post_id LIMIT 1", ARRAY_A );
//			$result = $this->_wpdb->get_row( "SELECT $column_name FROM $main_table WHERE ID=$post_id LIMIT 1", ARRAY_A );
//
//			return $result ? $result[ 0 ][ $column_name ] : [ ];
//		}

		// deprecated 271216
//		public function read_post_meta_data( $post_id, $meta_data_name ) {
//
//			$main_table = $this->_table_prefix . 'postmeta';
//
//			$result = $this->_wpdb->get_results( "SELECT meta_value FROM $main_table WHERE post_id=$post_id AND meta_key='$meta_data_name' LIMIT 1", ARRAY_A );
//
//			return $result ? $result[ 0 ][ 'meta_value' ] : [ ];
//		}

		public function read_meta_data( $post_id, $record_ids, $meta_columns ) {

			$options_table	 = $this->_table_prefix . 'options';
			$data = array();

			foreach ( $meta_columns as $column ) {
				foreach( $record_ids as $rec_id ) {
					
					$value = get_post_meta( $rec_id, $column, true );
					
					if ( $value ) {
						array_push( $data, $this->make_meta_object( $column, $value, $rec_id ) );
					} else {
						$alt_val = maybe_unserialize( get_post_meta( $rec_id, '_product_attributes', true ) );
						$col_name = str_replace( ' ', '-', strtolower($column));
						if ( $alt_val && isset( $alt_val[$col_name] ) ) {
							array_push( $data, $this->make_meta_object( $column, $alt_val[$col_name]['value'], $rec_id ) );
						}
					}
				}
			}
			
			$main_url = $this->_wpdb->get_var( "SELECT option_value FROM $options_table WHERE option_name = 'siteurl'" );
			$this->polish_data( $data, $post_id, $main_url );

			return $data;
		}
		
		private function make_meta_object( $key, $value, $id ) {

			$obj = new stdClass();
			$obj->meta_key = $key;
			$obj->meta_value = $value;
			$obj->post_id = $id;
			
			return $obj;
		}

		private function polish_data( &$data, $main_post_id, $site_url ) {

			foreach ( $data as $row ) {

				// make sure the _wp_attached_file data contains a valide url
				if ( $row->meta_key === '_wp_attached_file' ) {

					$row->meta_value = $this->get_post_thumbnail_url( $main_post_id, 'large' );

					// if the _wp_attached_file data is not a valid url than add the url data
					if ( !filter_var( $row->meta_value, FILTER_VALIDATE_URL ) ) {
						$row->meta_value = $site_url . '/wp-content/uploads/' . $row->meta_value;
					}
				}
				
				// convert the time stamp format to a usable date time format for the feed
				if ( $row->meta_key === '_sale_price_dates_from' || $row->meta_key === '_sale_price_dates_to' ) {
					$row->meta_value = convert_price_date_to_feed_format( $row->meta_value ) ;
				}
			}
		}

		public function delete_feed( $feed_id ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->delete( $main_table, array( 'product_feed_id' => $feed_id ) );
		}

		public function delete_meta( $feed_id ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feedmeta';
			return $this->_wpdb->delete( $main_table, array( 'product_feed_id' => $feed_id ) );
		}

		public function read_metadata( $feed_id ) {

			if ( $feed_id ) {

				$main_table = $this->_table_prefix . 'feedmanager_product_feedmeta';
				return $this->_wpdb->get_results( "SELECT * FROM $main_table WHERE product_feed_id = $feed_id AND meta_key != 'category_mapping'", ARRAY_A );
			} else {
				return false;
			}
		}
		
		public function get_product_filter_query( $feed_id ) {

			if ( $feed_id ) {

				$main_table = $this->_table_prefix . 'feedmanager_product_feedmeta';
				return $this->_wpdb->get_results( "SELECT meta_value FROM $main_table WHERE product_feed_id = $feed_id AND meta_key = 'product_filter_query'", ARRAY_A );
			} else {
				return false;
			}
		}

		public function get_columns_from_post_table() {

			$main_table = $this->_table_prefix . 'posts';
			return $this->_wpdb->get_results( "SHOW COLUMNS FROM $main_table" );
		}

		public function get_custom_product_attributes() {

			$main_table = $this->_table_prefix . 'woocommerce_attribute_taxonomies';
			return $this->_wpdb->get_results( "SELECT attribute_name, attribute_label FROM $main_table" );
		}

		public function get_custom_product_fields() {

			$main_table	 = $this->_wpdb->postmeta;
			return $this->_wpdb->get_col( "SELECT DISTINCT meta_key FROM $main_table WHERE meta_key NOT BETWEEN '_' AND '_z' OR meta_key LIKE '_cpf_%' OR meta_key LIKE '_wpmr_%' ORDER BY meta_key" );
		}
		
		public function get_own_variable_product_attributes( $variable_id ) {
			
			$main_table	 = $this->_wpdb->postmeta;
			$wpmr_attributes = array();
			
			foreach( $this->_wpdb->get_results( "SELECT meta_key, meta_value FROM $main_table WHERE (meta_key LIKE '_cpf_%' OR meta_key LIKE '_wpmr_%') AND (post_id = $variable_id)" ) as $key => $row ) {
				$wpmr_attributes[$row->meta_key] = $row->meta_value;
			}
			
			return $wpmr_attributes;
		}
		
		public function get_all_product_attributes() {
			
			$main_table	 = $this->_wpdb->postmeta;
			return $this->_wpdb->get_results( "SELECT DISTINCT meta_value FROM $main_table WHERE meta_key = '_product_attributes'" );
		}

		public function get_current_feed_status( $feed_id ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->get_results( "SELECT status_id FROM $main_table WHERE product_feed_id = $feed_id" );
		}

		public function get_country_id( $country_code ) {

			$main_table = $this->_table_prefix . 'feedmanager_country';
			return $this->_wpdb->get_row( "SELECT country_id FROM $main_table WHERE name_short = '$country_code'" );
		}

		public function get_status_id( $status ) {

			$main_table = $this->_table_prefix . 'feedmanager_feed_status';
			return $this->_wpdb->get_row( "SELECT status_id FROM $main_table WHERE status = '$status'" );
		}

		public function update_current_feed_status( $feed_id, $new_status ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->update( $main_table, array( 'status_id' => $new_status ), array( 'product_feed_id' => $feed_id ) );
		}

		public function set_nr_feed_products( $feed_id, $nr_products ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->update( $main_table, array( 'products' => $nr_products ), array( 'product_feed_id' => $feed_id ) );
		}
		
		public function get_nr_feed_products( $feed_id ) {
			
			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->get_row( "SELECT products FROM $main_table WHERE product_feed_id = '$feed_id'" );
		}

		/**
		 * Updates a new feed in the product_feed table.
		 * 
		 * @param (int) $feed_id
		 * @param (int) $channel_id
		 * @param (int) $country_id
		 * @param (int) $source_id
		 * @param (string) $title
		 * @param (string) $url
		 * @param (int) $status
		 * @return (int) nr of affected rows
		 */
		public function update_feed( $feed_id, $channel_id, $country_id, $source_id, $title, $main_category, $incl_variations, $is_aggregator,
							   $url, $status, $schedule ) {
			
			$main_table = $this->_table_prefix . 'feedmanager_product_feed';

			$result = $this->_wpdb->update( $main_table, array(
				'channel_id'	 => $channel_id,
				'include_variations' => $incl_variations,
				'is_aggregator'	 => $is_aggregator,
				'country_id'	 => $country_id,
				'source_id'		 => $source_id,
				'title'			 => $title,
				'main_category'	 => $main_category,
				'url'			 => $url,
				'status_id'		 => $status,
				'schedule'		 => $schedule,
				'updated'		 => date( 'Y-m-d H:i:s' ),
			), array( 'product_feed_id' => $feed_id ), array(
				'%d',
				'%d',
				'%d',
				'%d',
				'%d',
				'%s',
				'%s',
				'%s',
				'%d',
				'%s',
				'%s',
			), array( '%d' ) );

			return $result;
		}

		public function update_feed_update_data( $feed_id, $feed_url ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->update( $main_table, array( 'updated' => date( 'Y-m-d H:i:s' ), 'url' => $feed_url ), array( 'product_feed_id' => $feed_id ), array( '%s', '%s' ), array( '%d' ) );
			//return $this->_wpdb->update( $main_table, array( 'updated' => date( 'Y-m-d H:i:s' ) ), array( 'product_feed_id' => $feed_id ), array( '%s' ), array( '%d' ) );
		}

		public function get_file_url_from_feed( $feed_id ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->get_row( "SELECT url FROM $main_table WHERE product_feed_id = $feed_id", ARRAY_A )[ 'url' ];
		}

		public function update_feed_file_status( $feed_id, $status ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';
			return $this->_wpdb->update( $main_table, array( 'status_id' => $status ), array( 'product_feed_id' => $feed_id ), array( '%d' ), array( '%d' ) );
		}

		public function update_meta_data( $feed_id, $meta_data ) {

			// TODO: Onderstaande proces is nog niet echt efficient. In plaats van het standaard verwijderen
			// van elke bestaande regel met het geselecteerde id nummer, zou ik eerst moeten kijken of
			// deze regel wel is gewijzigd. Indien een regel niet is gewijzigd zou ik hem ook niet moeten
			// verwijderen.

			$main_table = $this->_table_prefix . 'feedmanager_product_feedmeta';

			// first remove all meta data belonging to this feed except the product_filter_query
			//$this->_wpdb->delete( $main_data, array( 'product_feed_id' => $feed_id ) ); // could not use it because it can't work with !=
			$this->_wpdb->query( "DELETE FROM $main_table WHERE product_feed_id = $feed_id AND meta_key != 'product_filter_query'" );

			$counter = 0;

			for ( $i = 0; $i < count( $meta_data ); $i ++ ) {

				if ( !empty( $meta_data[ $i ]->value ) && $meta_data[ $i ]->value !== '{}' ) {
					$result = $this->_wpdb->insert( $main_table, array(
						'product_feed_id'	 => $feed_id,
						'meta_key'			 => $meta_data[ $i ]->key,
						'meta_value'		 => $meta_data[ $i ]->value
					), array(
						'%d',
						'%s',
						'%s'
					) );

					$counter += $result;
				}
			}

			return $counter;
		}
		
		public function store_feed_filter( $feed_id, $filter ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feedmeta';
			
			if ( $filter ) {
				
				$exists = $this->_wpdb->get_results( "SELECT meta_id FROM $main_table WHERE product_feed_id = $feed_id AND meta_key = 'product_filter_query'" );
				
				if ( $exists ) {				
					$this->_wpdb->update( $main_table, array( 'meta_value' => $filter ), array( 'product_feed_id' => $feed_id, 'meta_key' => 'product_filter_query' ), array( '%s' ), array( '%d', '%s' ) );
				} else {
					$this->_wpdb->insert( $main_table, array( 'product_feed_id' => $feed_id, 'meta_key' => 'product_filter_query', 'meta_value' => $filter ), array( '%d', '%s', '%s' ) );
				}
			} else {
				$this->_wpdb->query( "DELETE FROM $main_table WHERE product_feed_id = $feed_id AND meta_key = 'product_filter_query'" );
			}
		}

		public function insert_meta_data( $feed_id, $meta_data, $category_mapping ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feedmeta';

			$counter = 0;

			for ( $i = 0; $i < count( $meta_data ); $i ++ ) {
				
				$result = $this->_wpdb->insert( $main_table, array(
						'product_feed_id'	 => $feed_id,
						'meta_key'			 => $meta_data[ $i ]['meta_key'],
						'meta_value'		 => $meta_data[ $i ]['meta_value']
					), array(
						'%d',
						'%s',
						'%s'
					) );

				$counter += $result;
			}

			$counter += $this->_wpdb->insert( $main_table, array(
					'product_feed_id'	 => $feed_id,
					'meta_key'			 => 'category_mapping',
					'meta_value'		 => $category_mapping[0]['meta_value']
				), array(
					'%d',
					'%s',
					'%s'
				) );

			return $counter;
		}
		
		public function title_exists( $feed_title ) {
			
			$main_table = $this->_table_prefix . 'feedmanager_product_feed';

			$count = $this->_wpdb->get_var( "SELECT COUNT(*) FROM $main_table WHERE title = '$feed_title'" );
			
			return $count > 0 ? true : false;
		}

		/**
		 * Inserts a new feed in the product_feed table and returns its new id.
		 * 
		 * @param int $channel_id
		 * @param int $country_id
		 * @param int $source_id
		 * @param string $title
		 * @param int $incl_variations
		 * @param int $is_aggregator
		 * @param string $url
		 * @param string $status
		 * @param string $schedule
		 * @return integer containing the id of the new feed
		 */
		public function insert_feed( $channel_id, $country_id, $source_id, $title, $main_category, $incl_variations, $is_aggregator, $url,
							   $status, $schedule ) {

			$main_table = $this->_table_prefix . 'feedmanager_product_feed';

			$this->_wpdb->insert( $main_table, array(
				'channel_id'	 => $channel_id,
				'include_variations' => $incl_variations,
				'is_aggregator'	 => $is_aggregator,
				'country_id'	 => $country_id,
				'source_id'		 => $source_id,
				'title'			 => $title,
				'main_category'	 => $main_category,
				'url'			 => $url,
				'status_id'		 => $status,
				'schedule'		 => $schedule,
				'updated'		 => date( 'Y-m-d H:i:s' ),
				'products'		 => 0
			), array(
				'%d',
				'%d',
				'%d',
				'%d',
				'%d',
				'%s',
				'%s',
				'%s',
				'%d',
				'%s',
				'%s',
				'%d'
			) );

			return $this->_wpdb->insert_id;
		}

	}

	

	

     // End of WPPFM_Queries class

endif;